/* This file is part of The_Merry_Turnip
 * 
 * The_Merry_Turnip - TODO: Description
 * Copyright © DAVY Guillaume 2009 <davyg2@gmail.com>
 * 
 * The_Merry_Turnip is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The_Merry_Turnip is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*!	\file chap.cpp
	\brief Implement chap class
*/

#include <game.h>

/*!
	\class chap
	This class contain mainly the item list (chap::listItem) of the chapter.\n
	When you call the init (chap::init()) function it will read an xml containing
	a list of %item and store them in the class. \n
	If you want to know more about chapter and how the xml must be formated, see chap.xml
	
	\todo add a name to chapter and maybe other stuff like description
	
	\file chap.xml
	\ingroup xml
	\section Description
	A chapter contain all the %item that the user can handle in his inventory.
	For example : paper, pen, keys, magasine, tools, books, ladder...
	The item are common to all location, that's why they are in the chapter.\n
	The last thing a chapter handle is the first location you are when begin
	the chapter(the changing of location are handled by other way)\n
	\n
	\section Xml
	So now the xml,\n
	I assume that you know how general xml are formated(tags system like html)\n
	\n
	The xml should begin whith a line like that :
	\code <?xml version="1.0"?>	\endcode \n
	Next there is three types of element possible :
		- \e chapter : this is the main element it takes as attribute :
			- \e nbr : the numberof item in the chapter
			
		\code <chapter nbr="2"> \endcode
		
		- \e %item : create an element it takes as attribute :
			- \e id : the id of the item
			- \e texture : the texture path of the item
			- \e combi the : id of the item it can be combine with(-1 if none)
			- \e combiResult : the result of a combinaison(-1 if none)
			
		\code <item id="1" texture="icon/ladder.png" combi="-1" combiResult="-1"> \endcode
		
		- \e %lieu : reference to the first location, it takes as attribute :
			- \e id : the id of the lieu
			
		\code <lieu id="0"/> \endcode
	\n
	
	\section Example
	
	To see a complete example just look at the source code of this file\n
	The example should create 2 items : a key and a ladder(according to the texture name)\n
	And the chapter should begin at location 3
*/

//! Construct the chap class, call base::base() and create chap::listItem
chap::chap() : base("chap")
{
	listItem = new std::vector<item>;
}

//! Destruct the chap class, delete chap::listItem
chap::~chap()
{
	delete listItem;
}

/** Call base::init() and read xmlChap 
	\param Id Id of the chap
	\param c Pointer to the core
	\param xmlChap Pointer to the IXMLReader you want to load
	\return The first lieu of the Chapter if succesful, -1 instead*/
int chap::init(int Id,coeur* c,IXMLReader* xmlChap)
{
	int l;// Will contain the number of the first lieu
	
	l=0;
	
	//Check that xmlChap is not equal to zero
	if (!xmlChap)
		return -1;
	
	//Init the base(no need the return value, which is the logId)
	base::init(Id,c);
	
	log(stringc("Chap xml : ") + stringc(Id) + " loading ...");
	while(xmlChap && xmlChap->read())
	{
		if (xmlChap->getNodeType()==EXN_ELEMENT)
		{
			//chapter element(main), contain the nbr of item
			if(core::stringw("chapter") == xmlChap->getNodeName())
				listItem->reserve(xmlChap->getAttributeValueAsInt(L"nbr"));
			//item element, contain the id, texture, combi and combiresult
			else if(core::stringw("item") == xmlChap->getNodeName())
			{
				//! \todo replace the log by a warning function
				if ( (xmlChap->getAttributeValueAsInt(L"combi")>-1) && (xmlChap->getAttributeValueAsInt(L"combiResult")<0))
					log( stringc("Warning : the item") + xmlChap->getAttributeValue(L"id") + " can be combine but has no result, it is ignore");
				addItem(//Call addItem to add this item to the list
					xmlChap->getAttributeValueAsInt(L"id"),
					stringc(xmlChap->getAttributeValue(L"texture")),
					xmlChap->getAttributeValueAsInt(L"combi"),
					xmlChap->getAttributeValueAsInt(L"combiResult")
				);
				log( stringc("New item : id:") + xmlChap->getAttributeValue(L"id") + ", texture: " + xmlChap->getAttributeValue(L"texture") + ", combi: " + xmlChap->getAttributeValue(L"combi") + ", combiResult: " + xmlChap->getAttributeValue(L"combiResult"));
			}
			//Contain the first lieu of the chap
			else if (core::stringw("lieu") == xmlChap->getNodeName())
			{
				l = xmlChap->getAttributeValueAsInt(L"id");
				log(stringc("Chosen lieu is ") + stringc(l));
			}
		}
	}
	return l;
}

/** Does nothing, apart from slowing the game
	\return always 0*/
int chap::update()
{
	return 0;
}

/** Remove all the item from chap::listItem and call base::close()
	\return base::close()*/
int chap::close()
{
	log("Closing each item :");
	while (! listItem->empty())//Loop until listItem is empty
	{
		log(stringc("Deleting item ") + stringc(listItem->back().getId()));
		listItem->pop_back();//Eemove the pointer from listItem
	}
	return base::close();//Close the base
}

/** Create the item, init it (item::init()) and add it to the chap::listItem
	\param Id Id of the item
	\param tex Pointer to the irrlicht texture corresponding to the item
	\param Combi Id of the item whith which this one can be combine
	\param CombiResult Id of the item resulting from the combinaison
	\return Pointer to the item created if succesful, 0 instead*/
item chap::addItem(int Id,stringc tex,int Combi,int CombiResult)
{
	item Item;
	if (Item.init(Id,tex,Combi,CombiResult))//Init the item
		return item();
	listItem->push_back(Item);//Add it ti the listItem
	log(stringc("item ") + stringc(Id) + "added");
	return Item;
}

/** Check for all element of chap::listItem if the id correspond to Id
	\param Id Id of the item to search
	\return The item if succesful, 0 instead*/
item chap::getItemFromId(int Id)
{
	for (unsigned int i=0;i<listItem->size();i++)//Loop on the listItem
		if (listItem->at(i).getId() == Id)//If the id correspond
			return listItem->at(i);//return the item
	return item();
}

/** This method is called when an event has to be treated, but this class
	do not wait event so this function does nothing
	\param event The event to parse
	\return Always false*/
bool chap::OnEvent(const SEvent& event)
{
	return false;
}

