/* This file is part of The_Merry_Turnip
 * 
 * The_Merry_Turnip - TODO: Description
 * Copyright © DAVY Guillaume 2009 <davyg2@gmail.com>
 * 
 * The_Merry_Turnip is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The_Merry_Turnip is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*!	\file modPyIrr.cpp
	\brief Create the python module who wrap the irrlicht needed class
*/

#include <game.h>

template<class cusString>
struct cusString_python
{
	cusString_python()
	{
		boost::python::converter::registry::push_back(&convertible,&construct,boost::python::type_id<cusString>());
	}
	
	static PyObject* convert(cusString const& s)
	{
		return boost::python::incref(boost::python::object(stringc(s).c_str()).ptr());
	}
	
	static void* convertible(PyObject* obj_ptr)
	{
		if (!PyString_Check(obj_ptr)) return 0;
			return obj_ptr;
	}

	static void construct(PyObject* obj_ptr,boost::python::converter::rvalue_from_python_stage1_data* data)
	{
		const char* value = PyString_AsString(obj_ptr);
		if (value == 0) boost::python::throw_error_already_set();
			void* storage = ((boost::python::converter::rvalue_from_python_storage<cusString>*)data)->storage.bytes;
			new (storage) cusString(value);
			data->convertible = storage;
	}
};

stringc reprPos(position2d<s32>* pos)
{
	return stringc("pos2d<X=") + stringc(pos->X) + ",Y=" + stringc(pos->Y) + ">";
}

stringc reprDim(dimension2d<s32>* dim)
{
	return stringc("dim2d<W=") + stringc(dim->Width) + ",H=" + stringc(dim->Height) + ">";
}

BOOST_PYTHON_MODULE(pyIrr)
{
	to_python_converter< stringw,cusString_python<stringw> >();
	cusString_python<stringw>();
	to_python_converter< stringc,cusString_python<stringc> >();
	cusString_python<stringc>();
	;
	class_< position2d<s32> >("pos2d")
		.def(init<s32,s32>())
		.add_property("X", &position2d<s32>::X )
		.add_property("Y", &position2d<s32>::Y )
		.def("__repr__", &reprPos)

	;
	class_< dimension2d<s32> >("dim2d")
		.def(init<s32,s32>())
		.add_property("W", &dimension2d<s32>::Width )
		.add_property("H", &dimension2d<s32>::Height )
		.def("__repr__", &reprDim)
	;
}

